use 5.006002; # encourage upgrade
use strict;
use warnings;
our $VERSION = sprintf "%d.%02d", q$Revision: 2.1 $ =~ /(\d+)/g;
no strict   'refs';
no warnings 'redefine';

sub UNIVERSAL::singleton_method { 
    my ($object, $method, $subref) = @_; 
    my $parent_class = ref $object; 
    my $new_class = "_Singletons::".(0+$object); 
    *{$new_class."::".$method} = $subref; 
    if ($new_class ne $parent_class) {
        @{$new_class."::ISA"} = ($parent_class); 
        bless $object, $new_class; 
    }
} 

1;
__END__
# Below is stub documentation for your module. You better edit it!

=head1 NAME

UNIVERSAL::SingletonMethod - makes singleton method univasally available

=head1 SYNOPSIS

  my $a = Some::Class->new; 
  my $b = Some::Class->new; 
     
  $a->singleton_method( dump => sub { 
      my $self = shift; 
      require Data::Dumper; 
      print STDERR Date::Dumper::Dumper($self)  
  }); 
     
  $a->dump; # Prints a representation of the object. 
  $b->dump; # Can't locate method "dump" 

=head1 DESCRIPTION

This module provides a Perl implementation of singleton methods which is further discussed in L<Class::SingletonMethod>.

=head1 CAVEAT

As the name suggests, this module hooks C<singleton_method> to package
UNIVERSAL.  Though smart, this might cause unexpected side-effect.  If
you want something more sound, Try L<Class::SingletonMethod::Std> or
L<Class::SingletonMethod::Hash>.

Also note this module implements singleton method by singleton
"class".  That is, rebless the object with a new class of its own with
the original class being its base.  So if you have 256 objects with
C<< ->singleton_method >> applied, you have 256 distinct namespace for
that.

If this section does not make sense, consult Advanced Perl
Programming, 2nd ed.

=head1 AUTHOR

Simon Cozens, C<simon@cpan.org>

Dan Kogai, C<dankogai@dan.co.jp>

=head1 SEE ALSO

ruby(1), L<Class::SingletonMethod::Std>, L<Class::SingletonMethod::Hash>

Advanced Perl Programming, 2nd ed.
L<http://www.oreilly.com/catalog/advperl2/>

=cut
